# Generated by Django 3.0.2 on 2020-04-15 04:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('otp_email', '0002_sidechanneldevice_email'),
    ]

    operations = [
        migrations.AddField(
            model_name='emaildevice',
            name='email',
            field=models.EmailField(blank=True, help_text='Optional alternative email address to send tokens to', max_length=254, null=True),
        ),
    ]
